package ui;

import game.Board;
import game.BoardListener;
import game.PieceLocation;
import game.checkers.Checker;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JComponent;

/**
 * ComponentBoardView is a component that displays a checkers game
 * by representing each checker with its own JComponent object.
 */
public class ComponentBoardView extends BoardBackground {
    // maps Checker to CheckerComponent
    private Map components = new HashMap();
    
    /**
     * Make a ComponentBoardView.
     * @param board board to display
     */
    public ComponentBoardView(Board board) {
        super(board);
        
        for (PieceLocation loc : board.getPieceLocations()) {
            addNewChecker(loc);
        }
        
        board.addBoardListener(new BoardListener() {
            public void pieceAdded(PieceLocation newLoc) {
                addNewChecker(newLoc);
            }
            public void pieceMoved(PieceLocation fromLoc, PieceLocation toLoc) {
                Checker checker = (Checker) toLoc.getPiece();
                CheckerComponent cc = (CheckerComponent) components.get(checker);
                if (cc == null)
                    throw new RuntimeException("got pieceMoved for an unknown piece: " + checker);
                placeOnSquare(cc, toLoc.getRow(), toLoc.getColumn());
            }
            public void pieceRemoved(PieceLocation oldLoc) {
                Checker checker = (Checker) oldLoc.getPiece();
                CheckerComponent cc = (CheckerComponent) components.get(checker);
                if (cc == null)
                    throw new RuntimeException("got pieceRemoved for an unknown piece: " + checker);
                remove(cc);
                repaint(cc.getBounds());
                components.remove(checker);
            }
        });
    }

    private void addNewChecker(PieceLocation loc) {
        Checker checker = (Checker) loc.getPiece();
        CheckerComponent cc = new CheckerComponent(checker);
        add(cc);
        components.put(checker, cc);
        placeOnSquare(cc, loc.getRow(), loc.getColumn());
    }
    
    private void placeOnSquare(CheckerComponent cc, int r, int c) {
        cc.setLocation(c*PPS, r*PPS);        
    }
    
    private class CheckerComponent extends JComponent {
        Checker checker;
        
        public CheckerComponent(Checker checker) {
            this.checker = checker;
            setPreferredSize(new Dimension(PPS, PPS));
            setSize(getPreferredSize());
        }
        
        public void paintComponent(Graphics g) {
            Dimension d = getSize();
            
            g.setColor((checker.getColor() == Checker.Color.RED) 
                    ? Color.RED : Color.BLACK);
            g.fillOval(2, 2, d.width-4, d.height-4);
        }
    }
    
}
