package ui;
import game.Board;
import game.checkers.Checker;

import java.awt.*;

import javax.swing.ImageIcon;


/**
 * PixelBoardView is a component that displays a checkers game
 * by drawing each checker as an image.  Red checkers are displayed
 * as red.gif, black checkers as black.gif.
 */
public class PixelBoardView extends StrokeBoardView {

    // checker images
    private ImageIcon red;
    private ImageIcon black;
    
    /**
     * Make a PixelBoardView.
     * 
     * @param board board to display
     */
    public PixelBoardView(Board board) {
        super(board);
        
        black = new ImageIcon(this.getClass().getResource("black-piece.png"));
        red = new ImageIcon(this.getClass().getResource("red-piece.png"));
    }
    
    /**
     * Paint a single checker.
     * @param g graphics context
     * @param piece Checker to paint
     * @param r row number, in range [0,board.size-1]
     * @param c column number, in range [0,board.size-1]
     * @effects Paints checker in square [r,c]
     */    
    protected void paintChecker(Graphics g, Checker piece, int r, int c) {
        ImageIcon img = (piece.getColor() == Checker.Color.RED)
            ? red : black;
        
        g.drawImage(img.getImage(), c * PPS, r * PPS, null);
    }
}
