package ui;
import game.Board;
import game.BoardListener;
import game.PieceLocation;
import game.checkers.Checker;

import java.awt.*;


/**
 * StrokeBoardView is a component that displays a checkers game
 * by drawing each checker as strokes.
 */
public class StrokeBoardView extends BoardBackground {

    /**
     * Make a StrokeBoardView
     * @param board board to display
     */
    public StrokeBoardView(Board board) {
        super(board);
        board.addBoardListener(new BoardListener() {
            public void pieceAdded(PieceLocation newLoc) {
                repaint();
            }
            public void pieceMoved(PieceLocation fromLoc, PieceLocation toLoc) {
                repaint();
            }
            public void pieceRemoved(PieceLocation oldLoc) {
                repaint();
            }
        });
    }
    
    /**
     * Paint this component.
     * @param g graphics context on which to paint
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        
        int n = board.getSize(); 
        Rectangle clip = g.getClip().getBounds();
        //System.err.println(clip);
        int r1 = Math.max(clip.y / PPS, 0);
        int c1 = Math.max(clip.x / PPS, 0);
        int r2 = Math.min(n-1, (clip.y + clip.height + PPS - 1) / PPS);
        int c2 = Math.min(n-1, (clip.x + clip.width + PPS - 1) / PPS);
        //System.err.println("from " + r1 + "," + c1 + " to " + r2 + "," + c2); 
        for (int r = r1; r <= r2; ++r) {
            for (int c = c1; c <= c2; ++c) {
                Checker piece = (Checker) board.getPieceAt(r, c);
                if (piece != null)
                    paintChecker(g, piece, r, c);
            }
        }
    }

    /**
     * Paint a single checker.
     * @param g graphics context
     * @param piece Checker to paint
     * @param r row number, in range [0,board.size-1]
     * @param c column number, in range [0,board.size-1]
     * @effects Paints checker in square [r,c]
     */
    protected void paintChecker(Graphics g, Checker piece, int r, int c) {
        g.setColor((piece.getColor() == Checker.Color.RED) 
                    ? Color.RED : Color.BLACK);
        g.fillOval(c*PPS+2, r*PPS+2, PPS-4, PPS-4);
        g.setColor(Color.BLACK);
        g.drawOval(c*PPS+1, r*PPS+1, PPS-3, PPS-3);
    }
}
